/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import com.rjconsultores.ventaboletos.dao.RutaDAO;
import com.rjconsultores.ventaboletos.dao.TarifaDAO;
import com.rjconsultores.ventaboletos.entidad.Categoria;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Marca;
import com.rjconsultores.ventaboletos.entidad.Moneda;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.entidad.RutaCombinacion;
import com.rjconsultores.ventaboletos.entidad.Tarifa;
import com.rjconsultores.ventaboletos.entidad.TipoPuntoVenta;
import com.rjconsultores.ventaboletos.entidad.Tramo;
import com.rjconsultores.ventaboletos.entidad.VigenciaTarifa;
import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

/**
 * 
 * @author rodrigo
 */
@Repository("tarifaDAO")
public class TarifaHibernateDAO extends GenericHibernateDAO<Tarifa, Integer>
		implements TarifaDAO {

	@Autowired
	private RutaDAO rutaDAO;
	private static Logger log = Logger.getLogger(TarifaHibernateDAO.class);

	@Autowired
	public TarifaHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<Tarifa> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return c.list();
	}

	public List<Tarifa> pesquisar(Marca marca, Tramo tramo, Moneda moneda,
			ClaseServicio claseServicio, VigenciaTarifa vigenciaTarifa) {

		Criteria c = getSession().createCriteria(getPersistentClass());

		if (marca != null) {
			c.add(Restrictions.eq("marca", marca));
		}

		if (tramo != null) {
			c.add(Restrictions.eq("tramo", tramo));
		}

		if (moneda != null) {
			c.add(Restrictions.eq("moneda", moneda));
		}

		if (claseServicio != null) {
			c.add(Restrictions.eq("claseServicio", claseServicio));
		}

		if (vigenciaTarifa != null) {
			c.add(Restrictions.eq("vigenciaTarifa", vigenciaTarifa));
		}

		return c.list();
	}

	public boolean existe(Marca marca, Tramo tramo, Moneda moneda, ClaseServicio claseServicio, VigenciaTarifa vigenciaTarifa) {

		Criteria c = getSession().createCriteria(getPersistentClass());

		c.add(Restrictions.eq("marca", marca));
		c.add(Restrictions.eq("tramo", tramo));
		c.add(Restrictions.eq("moneda", moneda));
		c.add(Restrictions.eq("claseServicio", claseServicio));
		c.add(Restrictions.eq("vigenciaTarifa", vigenciaTarifa));
		c.setProjection(Projections.rowCount());

		return (HibernateFix.count(c.list()) > 0);
	}

	public List<Tarifa> pesquisar(Marca marca, Parada origem, Parada destino, Moneda moneda,
			ClaseServicio claseServicio, VigenciaTarifa vigenciaTarifa,
			Categoria categoria, TipoPuntoVenta tipoPuntoVenta) {

		Criteria c = getSession().createCriteria(getPersistentClass());
		c.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);

		Criteria cCategoria = null;
		if (categoria != null) {
			cCategoria = c.createCriteria("lsTarifaCategoria");
			cCategoria.add(Restrictions.eq("categoria", categoria));
			cCategoria.add(Restrictions.eq("activo", Boolean.TRUE));
		}

		Criteria cTipoPuntoVenta = null;
		if (tipoPuntoVenta != null) {
			cTipoPuntoVenta = c.createCriteria("lsTarifaTipoptovta");
			cTipoPuntoVenta.add(Restrictions.eq("tipoPuntoVenta", tipoPuntoVenta));
			cTipoPuntoVenta.add(Restrictions.eq("activo", Boolean.TRUE));
		}

		if (marca != null) {
			c.add(Restrictions.eq("marca", marca));
		}

		Criteria cTramo = c.createCriteria("tramo");
		if (origem != null) {
			cTramo.add(Restrictions.eq("origem", origem));
		}

		if (destino != null) {
			cTramo.add(Restrictions.eq("destino", destino));
		}

		if (moneda != null) {
			c.add(Restrictions.eq("moneda", moneda));
		}

		if (claseServicio != null) {
			c.add(Restrictions.eq("claseServicio", claseServicio));
		}

		if (vigenciaTarifa != null) {
			c.add(Restrictions.eq("vigenciaTarifa", vigenciaTarifa));
		}

		return c.list();
	}

	public Tarifa buscar(Tramo tramo, Marca marca, ClaseServicio claseServicio,
			VigenciaTarifa vigenciaTarifa, Moneda moneda) {

		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("marca", marca));

		Criteria cTramo = c.createCriteria("tramo");
		cTramo.add(Restrictions.eq("activo", true));

		c.add(Restrictions.eq("tramo", tramo));
		c.add(Restrictions.eq("claseServicio", claseServicio));
		c.add(Restrictions.eq("vigenciaTarifa", vigenciaTarifa));
		c.add(Restrictions.eq("moneda", moneda));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return (Tarifa) c.uniqueResult();
	}

	public List<Tarifa> buscarPorVigencia(VigenciaTarifa vigencia) {
		Criteria c = getSession().createCriteria(getPersistentClass());

		if (vigencia != null) {
			c.add(Restrictions.eq("vigenciaTarifa", vigencia));
		}

		return c.list();
	}

	public List<Tarifa> pesquisar(List<Moneda> lsMoneda, List<Marca> lsMarca,
			List<ClaseServicio> lsClaseServicio, List<Categoria> lsCategoria,
			List<Empresa> lsEmpresa, List<Ruta> lsRuta,
			List<TipoPuntoVenta> lsTipoPuntoVenta, List<VigenciaTarifa> lsVigenciaTarifa) {
		// Criteria c = getSession().createCriteria(getPersistentClass());
		// c.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		//
		// c.add(Restrictions.eq("activo", Boolean.TRUE));
		//
		// if (!lsVigenciaTarifa.isEmpty()) {
		// List<VigenciaTarifa> lsAux = null;
		// if (lsVigenciaTarifa.size() > 1000) {
		// lsAux = new ArrayList<VigenciaTarifa>();
		// for (VigenciaTarifa v : lsVigenciaTarifa) {
		// lsAux.add(v);
		// if (lsAux.size() == 1000) {
		// c.add(Restrictions.in("vigenciaTarifa", lsAux));
		// lsAux = new ArrayList<VigenciaTarifa>();
		// }
		// }
		// if (lsAux != null && lsAux.size() > 0) {
		// c.add(Restrictions.in("vigenciaTarifa", lsAux));
		// }
		// } else {
		// c.add(Restrictions.in("vigenciaTarifa", lsVigenciaTarifa));
		// }
		// }
		//
		// if (!lsMoneda.isEmpty()) {
		// c.add(Restrictions.in("moneda", lsMoneda));
		// }
		//
		// if (!lsMarca.isEmpty()) {
		// c.add(Restrictions.in("marca", lsMarca));
		// }
		//
		// if (!lsClaseServicio.isEmpty()) {
		// c.add(Restrictions.in("claseServicio", lsClaseServicio));
		// }
		//
		// Criteria cCategoria = null;
		// if (!lsCategoria.isEmpty()) {
		// cCategoria = c.createCriteria("lsTarifaCategoria");
		// cCategoria.add(Restrictions.in("categoria", lsCategoria));
		// }
		//
		// Criteria cTipoPuntoVenta = null;
		// if (!lsTipoPuntoVenta.isEmpty()) {
		// cTipoPuntoVenta = c.createCriteria("lsTarifaTipoptovta");
		// cTipoPuntoVenta.add(Restrictions.in("tipoPuntoVenta", lsTipoPuntoVenta));
		// }
		//
		// if (!lsEmpresa.isEmpty()) {
		// Criteria criteriaRuta = getSession().createCriteria(Ruta.class);
		// Criteria cEmpresa = criteriaRuta.createCriteria("lsRutaEmpresa");
		// log.info("Excel: Qtd lsEmpresa" + lsEmpresa.size());
		// cEmpresa.add(Restrictions.in("empresa", lsEmpresa));
		//
		// List<Ruta> lsRutasComEmpresa = criteriaRuta.list();
		// List<Tramo> lsTramos = new ArrayList<Tramo>();
		// for (Ruta ruta : lsRutasComEmpresa) {
		// ruta = rutaDAO.obtenerID(ruta.getRutaId());
		// for (RutaCombinacion rc : ruta.getRutaCombinacionList()) {
		// if (!lsTramos.contains(rc.getTramo())) {
		// lsTramos.add(rc.getTramo());
		// }
		// }
		// }
		//
		// if (!lsTramos.isEmpty()) {
		// List<Tramo> lsAux = null;
		// if (lsTramos.size() > 1000) {
		// lsAux = new ArrayList<Tramo>();
		// for (Tramo t : lsTramos) {
		// lsAux.add(t);
		// if (lsAux.size() == 500) {
		// log.info("Excel: Qtd lsAux 1 - " + lsAux.size());
		// c.add(Restrictions.in("tramo", lsAux));
		// lsAux = new ArrayList<Tramo>();
		// }
		// }
		// if (lsAux != null && lsAux.size() > 0) {
		// log.info("Excel: Qtd lsAux 2 - " + lsAux.size());
		// c.add(Restrictions.in("tramo", lsAux));
		// }
		// } else {
		// log.info("Excel: Qtd lsAux 3 - " + lsAux.size());
		// c.add(Restrictions.in("tramo", lsTramos));
		// }
		// }
		// }
		//
		//
		//
		//
		// if (!lsRuta.isEmpty()) {
		// List<Tramo> lsTramos = new ArrayList<Tramo>();
		// for (Ruta ruta : lsRuta) {
		// ruta = rutaDAO.obtenerID(ruta.getRutaId());
		// for (RutaCombinacion rc : ruta.getRutaCombinacionList()) {
		// if (!lsTramos.contains(rc.getTramo())) {
		// lsTramos.add(rc.getTramo());
		// }
		// }
		// }
		//
		// if (!lsTramos.isEmpty()) {
		// List<Tramo> lsAux = null;
		// if (lsTramos.size() > 1000) {
		// lsAux = new ArrayList<Tramo>();
		// for (Tramo t : lsTramos) {
		// lsAux.add(t);
		// if (lsAux.size() == 1000) {
		// c.add(Restrictions.in("tramo", lsAux));
		// lsAux = new ArrayList<Tramo>();
		// }
		// }
		// if (lsAux != null && lsAux.size() > 0) {
		// c.add(Restrictions.in("tramo", lsAux));
		// }
		// } else {
		// c.add(Restrictions.in("tramo", lsTramos));
		// }
		// }
		// }
		//
		// return c.list();

		StringBuilder hql = new StringBuilder(" SELECT T FROM ");
		hql.append(" Tarifa as T  ");
		hql.append("  inner join T.marca as M ");
		hql.append("  inner join T.moneda as MON ");
		hql.append("  inner join T.vigenciaTarifa as VT  ");
		hql.append("  inner join T.claseServicio as CS  ");
		hql.append("  ,RutaCombinacion as RC,RutaEmpresa as RE,Ruta as R ");
		hql.append(" WHERE T.activo = 1 AND RC.activo = 1 AND RE.activo = 1 AND R.activo = 1 ");
		hql.append(" AND RC.tramo = T.tramo ");
		hql.append(" AND RE.ruta = RC.ruta ");
		hql.append(" AND RE.ruta.rutaId = R.rutaId ");
		hql.append(" AND R.claseServicio = CS ");

		if (!lsVigenciaTarifa.isEmpty()) {
			hql.append(" AND VT in (:lsVigencia) ");
		}
		if (!lsEmpresa.isEmpty()) {
			hql.append(" AND RE.empresa  in (:lsEmpresa) ");
		}
		if (!lsRuta.isEmpty()) {
			hql.append("  AND RC.ruta in (:lsRuta)");
		}
		if (!lsMarca.isEmpty()) {
			hql.append("  AND M in (:lsMarca) ");
		}
		if (!lsMoneda.isEmpty()) {
			hql.append(" AND MON in (:lsMoneda) ");
		}
		if (!lsClaseServicio.isEmpty()) {
			hql.append(" AND CS in (:lsClaseServicio) ");
		}

		Query sq = getSession().createQuery(hql.toString());
		sq.setParameterList("lsVigencia", lsVigenciaTarifa);

		if (!lsEmpresa.isEmpty()) {
			sq.setParameterList("lsEmpresa", lsEmpresa);
		}
		if (!lsRuta.isEmpty()) {
			sq.setParameterList("lsRuta", lsRuta);
		}
		if (!lsMarca.isEmpty()) {
			sq.setParameterList("lsMarca", lsMarca);
		}
		if (!lsMoneda.isEmpty()) {
			sq.setParameterList("lsMoneda", lsMoneda);
		}
		if (!lsClaseServicio.isEmpty()) {
			sq.setParameterList("lsClaseServicio", lsClaseServicio);
		}
		return sq.list();

	}

	public List<Tarifa> buscarExisteVigencia(VigenciaTarifa vigenciaTarifa) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.add(Restrictions.eq("vigenciaTarifa", vigenciaTarifa));

		return c.list();
	}
}
