/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import com.rjconsultores.ventaboletos.dao.TramoDAO;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Tramo;
import com.rjconsultores.ventaboletos.entidad.TramoServicio;
import com.rjconsultores.ventaboletos.entidad.TramoTiempo;
import com.rjconsultores.ventaboletos.entidad.Via;
import com.rjconsultores.ventaboletos.service.TramoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author Administrador
 */
@Service("tramoService")
public class TramoServiceImpl implements TramoService {

    @Autowired
    private TramoDAO tramoDAO;

    public List<Tramo> obtenerTodos() {
        return tramoDAO.obtenerTodos();
    }

    public Tramo obtenerID(Integer id) {
        return tramoDAO.obtenerID(id);
    }

    @Transactional
    public Tramo suscribir(Tramo entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return tramoDAO.suscribir(entidad);
    }

    @Transactional
    public Tramo actualizacion(Tramo entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return tramoDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(Tramo entidad) {
        entidad = tramoDAO.obtenerID(entidad.getTramoId());


        if (entidad.getLsTramoTiempo() != null) {
            for (TramoTiempo tramo : entidad.getLsTramoTiempo()) {
                tramo.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
                tramo.setFecmodif(Calendar.getInstance().getTime());
                tramo.setActivo(Boolean.FALSE);
            }
        }

        if (entidad.getTramoServicioList() != null) {
            for (TramoServicio tramo : entidad.getTramoServicioList()) {
                tramo.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
                tramo.setFecmodif(Calendar.getInstance().getTime());
                tramo.setActivo(Boolean.FALSE);
            }
        }

        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);

        tramoDAO.actualizacion(entidad);
    }

    public Tramo obtenerPorOrigemDestino(Parada origem, Parada destino) {
        return tramoDAO.obtenerPorOrigemDestino(origem, destino);
    }

    public List<Tramo> obtenerListPorOrigemDestino(Parada origem, Parada destino) {
        return tramoDAO.obtenerListPorOrigemDestino(origem, destino);
    }

    public Tramo obtenerTramotPorOrigemDestinoVia(Parada origem, Parada destino, Via via) {
        return tramoDAO.obtenerTramotPorOrigemDestinoVia(origem, destino, via);
    }

    public Tramo buscar(Parada origem, Parada destino, Via via) {
        return tramoDAO.busca(origem, destino, via);
    }

    public List<Tramo> buscarCveParada(String origen, String destino, String via) {
        return tramoDAO.buscarCveParada(origen, destino, via);
    }

    public Tramo obtenerTramoUnicoRuta(Parada origem, Parada destino, Via via, ClaseServicio claseServicio) {
        return tramoDAO.obtenerTramoUnicoRuta(origem, destino, via, claseServicio);
    }

    public List<Tramo> buscar(List<Parada> lsParadas, Via via) {
        List<Tramo> respuesta = new ArrayList<Tramo>();

        Parada origen = lsParadas.get(0);
        Parada destino = null;


        for (int i = 1; i < lsParadas.size(); i++) {
            destino = lsParadas.get(i);
            Tramo busca = tramoDAO.busca(origen, destino, via);

            if (busca!= null) {
                respuesta.add(busca);
            }
        }

        return respuesta;
    }

    @Transactional
    public void guardarTramosKm(List<Tramo> lsTramos, Via via) {
        for (Tramo tramo : lsTramos) {
            Tramo busca = tramoDAO.busca(tramo.getOrigem(), tramo.getDestino(), via);
            if (busca==null) {
                tramo.setVia(via);
                tramo.setActivo(Boolean.TRUE);
                tramo.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
                tramo.setFecmodif(Calendar.getInstance().getTime());
                tramo.setDesctramo(tramo.getOrigem().getCveparada().trim()+"-"+tramo.getDestino().getCveparada().trim());
                tramoDAO.suscribir(tramo);
                
            } else {
                Tramo tramoExiste = busca;
                tramoExiste.setKmPagoConductor(tramo.getKmPagoConductor());
                tramoExiste.setKmReal(tramo.getKmReal());
                tramoExiste.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
                tramoExiste.setFecmodif(Calendar.getInstance().getTime());
            }
        }
    }
    public List<Via> obtenerViasOrigemDestino(Parada origem, Parada destino) {
    	return tramoDAO.obtenerViasOrigemDestino(origem, destino);
    }
    public List<Tramo> obtenerPorOrigemDestinoFetchLazy(Parada origem, Parada destino) {
    	return tramoDAO.obtenerPorOrigemDestinoFetchLazy(origem, destino);
    }
}
